'''
Created on 18 Jun 2019

'''

class BinHeapNode(object):
    '''
    A node that is used in our implementation of BinomialHeaps.
    
    Attributes
    ----------
    _children_sorted : list of BinHeapNode
        a list of all children of this node, sorted by non-decreasing order of index
    _data : int
        the current value of the node
    _index : int
        The subtree of the node corresponds to a binomial tree with the specified index.
    _parent : BinHeapNode
        the parent of the node

    '''

    def __init__(self, data):
        '''
        Constructor. Creates a new BinHeapNode which stores the specified data.
        
        Parameters
        ----------
        data : int
            The value of the node.
        '''
        self._children_sorted = list();
        self._data = data;
        self._index = 0;
        self._parent = None;
    
    def get_parent(self):
        '''
        Gets the parent node.
        
        Returns
        -------
        BinHeapNode
            the parent node
        '''
        return self._parent;
    
    def set_parent(self, parent):
        '''
        Sets the parent node to the given node.
        
        Parameters
        ----------
        parent : BinHeapNode
            the parent to set
        '''
        self._parent = parent;
    
    def get_data(self):
        '''
        Gets the data stored in the node.
        
        Returns
        -------
        int
            the data in the node
        '''
        return self._data;
    
    def set_data(self, data):
        '''
        Sets the data to the given node.
        
        Parameters
        ----------
        data : int
            the data to set
        '''
        self._data = data;
        
    def get_index(self):
        '''
        Gets the index of the node. An index of k means that the node is a root node of a k-th binomial tree.
        
        Returns
        -------
        int
            the index of the node
        '''
        return self._index;
    
    def set_index(self, index):
        '''
        Sets the index of the given node.
        
        Parameters
        ----------
        index : int
            the index to set
        '''
        self._index = index;
    
    def get_children_sorted(self):
        '''
        Gets the the list of children.
        
        Returns
        -------
        list of BinHeapNode
            the sorted list of children
        '''
        return self._children_sorted;
    
    def add_child(self, child):
        '''
        Adds a child in the correct position to the sorted list of children.
        
        Parameters
        ----------
        child : BinHeapNode
            the child to add
        '''    
        # TODO: Add your code here!
        
    def __str__(self):
        return_string = "[(" + str(self._data) + ")";
        if ((self._children_sorted is not None) and (len(self._children_sorted)>0)) :
            for node in self._children_sorted :
                return_string += node.__str__();
        return_string += "]";
        return return_string;